from SoftwarePilot.SoftwarePilotService import SoftwarePilotService
from SoftwarePilot.SoftwarePilotDocker import SoftwarePilotDocker
import socket
import importlib

class SoftwarePilot:
	'''
	A library to control drones and services through python scripts
	
	...
	
	Attributes
	----------
	drone_lib_dict : map{str:str}
		a map of all integrated drone libraries
	docker : SoftwarePilotDocker
		docker wrapper library for easier interaction with SoftwarePilot
	
	Methods
	-------
	setup_drone(drone_type, connection_method, download_dir)
		initiates and returns a drone object of the chosen drone type
	setup_docker()
		iniates a docker client
	setup_service(ip_adress, port)
		initiates and returns a SoftwarePilotService object that connects to the given web api
	get_host_ip()
		gets the host's ip address
	'''
	def __init__(self):
		self.drone_lib_dict = {"parrot-anafi" : "AnafiController"}
	
	def setup_drone(self, drone_type, connection_method, download_dir="None"):
		'''
		initiates and returns a drone object of the chosen drone type
		
		Parameters
		----------
		drone_type : DroneLibrary
			the drone type to be initiated
		connection_method : str
			the type of connection to be established to the drone
		download_dir : str, optional
			the download path for drone media (default = "None")
			
		Return
		----------
		drone : DroneLibrary
			the drone object
		'''
		
		try:
			if drone_type in self.drone_lib_dict.keys():
				drone_type = self.drone_lib_dict[drone_type]
				drone_lib = importlib.import_module("SoftwarePilot." + drone_type)
			else:	
				drone_lib = importlib.import_module(drone_type)
			drone_call = getattr(drone_lib, drone_type)
			drone = drone_call(connection_method, download_dir="None")
			return drone
		except:
			print (f"ModuleError: Invalid drone module '{drone_type}'")
			return None
	
	def setup_docker(self):
		'''
		iniates a docker client
		'''
		
		self.docker = SoftwarePilotDocker()
		
	def setup_service(self, ip_address = "172.0.0.1", port = 8000):
		'''
		initiates and returns a SoftwarePilotService object that connects to the given web api
		
		Parameters
		----------
		ip_address : str, optional
			the ip address to be used for the web api requests (default = "172.0.0.1")
		port : int, optional
			the port to be used for the web api requests (default = 8000)
		
		Return
		----------
		service : SoftwarePilotService
			the service object meant to interact with the rest api
		'''
		
		service = SoftwarePilotService(ip_address = ip_address, port = port)
		return service

	def get_host_ip(self):
		'''
		gets the host's ip address
		
		Return
		----------
		ip_address : str
			the host's ip address
		'''
		
		hostname = socket.gethostname()   
		ip_address = socket.gethostbyname(hostname)
		return ip_address



import os
import olympe
from SoftwarePilot.AnafiCamera import AnafiCamera
from SoftwarePilot.AnafiPiloting import AnafiPiloting
from SoftwarePilot.AnafiRTH import AnafiRTH
from olympe.messages.ardrone3.PilotingState import PositionChanged
from olympe.messages.obstacle_avoidance import set_mode, status

class AnafiController:
	''' 
	Parrot Olympe wrapper for controlling Parrot Anafi drones.
	
	...
	
	Attributes
	----------
	drone_ip : str
		the drone's ip address
	drone_rtsp_port : str
		the drone's rtsp port which connects to the live stream
	drone_url : str
		the url used request to make requests from the drone
	drone : olympe.Drone
		the drone object
	camera : AnafiCamera
		the drone camera method interface
	piloting : AnafiPiloting
		the drone flight controls method interface
	rth : AnafiRTH
		the drone Return From Home (RTH) methods interface	

	Methods
	-------
	connect()
		Establishes a connection with the drone
	disconnect()
		Breaks current connection with the drone
	get_drone_coordinates()
		Returns drone's current gps coordinates
	'''	
	
	def __init__(self, connection_type = 1, download_dir = "None"):
		'''
		Parameters
		----------
		connection_type : str or int, optional
			The connection type to the drone (default = 1)
			- 'physical' or 0
			- 'controller' or 1
		download_dir : str
			The location drone media will be downloaded (default = "None")
			If none is provided it will download them to /AnafiMedia
			If the directory does not exist it will be created
		'''
		if connection_type == "physical" or connection_type == 0:
			self.drone_ip = "192.168.42.1"		
			self.drone_rtsp_port = os.environ.get("DRONE_RTSP_PORT")
			self.drone_url = "http://{}/".format(self.drone_ip)

		elif connection_type == "controller" or connection_type == 1:
			self.drone_ip = "192.168.53.1"		
			self.drone_rtsp_port = os.environ.get("DRONE_RTSP_PORT", "554")
			self.drone_url = "http://{}:180/".format(self.drone_ip)
		else:
			raise RuntimeError("Illegal object parameter")

		self.drone = olympe.Drone(self.drone_ip)
		if download_dir == "None":
			if os.path.isdir("AnafiMedia") == False:			
				os.mkdir("AnafiMedia")		
			self.download_dir = "AnafiMedia"
		else:
			if os.path.isdir(download_dir) == False:
				os.mkdir(download_dir)
			self.download_dir = download_dir

		self.camera = AnafiCamera(self.drone, self.drone_ip, self.drone_rtsp_port, self.drone_url, self.download_dir)
		self.piloting = AnafiPiloting(self.drone)
		self.rth = AnafiRTH(self.drone)
		self.rth.setup_rth()
			
	def connect(self):
		'''
		Establishes a connection with the drone
		'''
		
		assert self.drone.connect(retry = 3)
		print("< Drone Connected >")
	
	def disconnect(self):
		'''
		Breaks the current connection with the drone 
		'''
		
		assert self.drone.disconnect()
		print("< Drone Disconnected >")
	
	def get_drone_coordinates(self):
		'''
		Returns the drone's current gps coordinates
		
		Return
		----------
		coordinates : str[latitude, longitude, altitude]
			list containing the current latitude, longitude, and altitude gps values
		'''
		
		latitude = self.drone.get_state(PositionChanged)["latitude"]
		longitude = self.drone.get_state(PositionChanged)["longitude"]
		altitude = self.drone.get_state(PositionChanged)["altitude"]
		coordinates = [latitude, longitude, altitude]
		return coordinates



import olympe
from olympe.messages.rth import ( 
	set_preferred_home_type,
	set_custom_location,
	set_auto_trigger_mode,
	set_delay,
	set_ending_behavior,
	set_ending_hovering_altitude,
	return_to_home,
	abort,
	cancel_auto_trigger,
)

class AnafiRTH:
	'''
	Wrapper for the Parrot Olympe Return To Home (RTH) methods
	
	...
	
	Attributes
	----------
	drone : olympe.Drone
		the drone object
		
	Methods
	-------
	setup_rth(home_type, gps_coordinates, auto_trigger, delay, ending_behavior, ending_hovering_altitude)
		Setup rth features
	return_to_home()
		Returns the drone to rth location
	abort_return_to_home()
		Stops rth call
	cancel_auto_trigger()
		Cancels auto trigger when connection is lost
	'''
	
	def __init__(self, drone_object):
		'''
		Parameters
		----------
		drone : olympe.Drone
			the drone object
		'''
		
		self.drone = drone_object
			
	def setup_rth(self, 
		home_type = "takeoff",
		gps_coordinates = "None",
		auto_trigger = "on",
		delay = 5,
		ending_behavior = "landing",
		ending_hovering_altitude = 2
	):
		'''
		Setup rth features
		
		Parameters
		----------
		home_type : str, optional
			the target location for rth
			- "takeoff"/"pilot"/"custom"
		gps_coordinates : str, optional
			the gps coordinates for a custom rth location
			must be entered in the format longitude,latitude,altitude (comma separated no spaces)
			is only used when the home_type is "custom"
		auto_trigger : str, optional
			If auto_trigger is "on", rth will be automatically triggerred when connection to the drone is lost
			- "on"/"off"
		delay : int, optional
			The delay before rth will be automatically triggered upon connection lost
			is only used if auto_trigger is "on"
		ending_behavior : str, optional
			the behavior after reaching home
			- "landing"/"hovering"
		ending_hovering_altitude : int, optional
			 the altitude at which to hover over the ground once home is reached
			 the value is interpreted as meters above ground
			 is only used if ending_behavior is "hovering"
		'''
	
		self.drone(set_preferred_home_type(home_type)).wait()
		if home_type == "custom":
			gps_coordinates = gps_coordinates.split(',')
			self.drone(set_custom_location(gps_coordinates[0], gps_coordinates[1], gps_coordinates[2])).wait()
		self.drone(set_auto_trigger_mode(auto_trigger)).wait()
		self.drone(set_delay(delay)).wait()
		self.drone(set_ending_behavior(ending_behavior)).wait()
		if ending_behavior == "hovering":
			self.drone(set_ending_hovering_altitude(ending_hovering_altitude)).wait()

	def return_to_home(self):
		'''
		Returns the drone to rth location
		'''
		
		self.drone(return_to_home()).wait()

	def abort_return_to_home(self):
		'''
		Stops rth call
		'''
		
		self.drone(abort()).wait()

	def cancel_auto_trigger(self):
		'''
		Cancels auto trigger when connection is lost
		'''
		
		self.drone(cancel_auto_trigger).wait()
		
		
		



import csv
import os
import queue
import threading
import cv2
import requests
import shutil
import olympe
from olympe.video.renderer import PdrawRenderer
from olympe.messages.camera import (
	set_camera_mode,
	set_recording_mode,
	set_streaming_mode,
	set_photo_mode,
	take_photo,
	stop_photo,
	photo_progress,
	start_recording,
	stop_recording,
	recording_progress,
)

# << Camera Photo, Recording and Stream Methods >>			
class AnafiCameraMedia:
	'''
	Wrapper for the Parrot Olympe camera methods involving media
	
	...
	
	Attributes
	----------
	drone : olympe.Drone
		the drone object
	drone_ip : str
		the drone's ip_address
	drone_rtstp_port : str
		the drone's rtsp port which connects to the live stream
	drone_url : str
		the url used request to make requests from the drone
	drone_media_api_url : str
		the complete url used to make media requests from the drone
	download_dir : str
		The location drone media will be downloaded
	camera_mode : str
		the drone's current camera mode (None, photo, recording, streaming)
		
	Methods
	-------
	setup_photo(mode, format, file_format, burst, bracketing, capture_interval)
		Prepares the drone camera for photos, and changes camera_mode to "photo"
	take_photo()
		Takes a photo
	start_lapse_photo()
		Starts to take time/gps lapse photos
	stop_lapse_photo()
		Stops current time/gps lapse photos
	setup_recording(mode, resolution, framerate, hyperlapse)
		Prepares the drone camera for recording, and changes camera_mode to "recording"
	start_recording()
		Starts a recording
	stop_recording()
		Stops current recording
	download_last_media()
		Downloads the last media taken
	setup_stream(value, record, yuv_frame_processing, yuv_frame_cb, h264_frame_cb, start_cb, end_cb, flush_cb)
		Prepares the drone camera for streaming, and changes camera_mode to "streaming"
	start_stream()
		Starts the live video stream
	strop_stream()
		Stops the current live video stream
	'''

	def __init__(self, drone_object, drone_ip, drone_rtsp_port, drone_url, download_dir):
		'''
		Parameters
		----------
		drone_object : olympe.Drone
			the drone object
		drone_ip : str
			the drone's ip_address
		drone_rtstp_port : str
			the drone's rtsp port which connects to the live stream
		drone_url : str
			the url used request to make requests from the drone
		download_dir : str
			The location drone media will be downloaded
		'''
		
		self.drone = drone_object
		self.drone_ip = drone_ip		
		self.drone_rtsp_port = drone_rtsp_port
		self.drone_url = drone_url
		self.drone_media_api_url = self.drone_url + "api/v1/media/medias/"

		self.download_dir = download_dir
		self.camera_mode = "None"
	
	# << Photo Methods >>
	def setup_photo(self,
		mode = "single",
		format= "rectilinear",
		file_format="dng",
		burst="burst_14_over_1s",
		bracketing="preset_1ev",
		capture_interval=0.1
	):
		'''
		Prepares the drone camera for photos, and changes camera_mode to "photo"
		
		Parameters
		----------
		mode : str, optional
			the photo mode (default = "single")
			- "single"/"bracketing"/"burst"/"time_lapse"/"gps_lapse"
		format : str, optional
			the photo format (default = "rectilinear")
			- "full_frame"/"rectilinear"
		file_format : str, optional
			the photo file format (default = "dng")
			- "jpeg"/"dng"/"dng_jpeg"
		burst : str, optional
			The amount of photos/second to take (default = "burst_14_over_1s")
			only used if mode is "burst"
			- "burst_X_over_Xs" (14/10/4 , 4/2/1)
		bracketing : str, optional
			The exposure values of the bracketed photos (default = "preset_1ev")
			only used if mode is "bracketing"
			- preset_Xev (1/2/3)
			- preset_Xev_Xev (1 , 2/3)/(2 , 3)
			- preset_Xev_Xev_Xev (1 , 2 , 3)
		capture_interval : float, optional
			The interval between each photo (default = 0.1)
			seconds in time lapse and meters in gps lapse
			only used if mode is "time_lapse" or "gps_lapse"
		'''
	
		self.drone(set_camera_mode(cam_id=0, value="photo")).wait()
		self.drone(set_photo_mode(
			cam_id=0,
			mode= mode,
			format= format,
			file_format= file_format,
			burst= burst,
			bracketing= bracketing,
			capture_interval= capture_interval,
		)).wait()
		self.camera_mode = "photo"
		print("< Photo Mode >")
	
	def take_photo(self):
		'''
		Takes a photo
		'''
			
		self.media_saved = self.drone(photo_progress(result="photo_saved", _policy="wait"))
		self.drone(take_photo(cam_id=0)).wait()
		self.media_saved.wait()
		print("< Take Photo >")
	
	def start_lapse_photo(self):
		'''	
		Starts to take time/gps lapse photos. Continues to take pictures until "stop_lapse_photo" is called
		Should only be used when camera mode is set to eiher "time_lapse" or "gps_lapse"
		'''
		
		self.media_saved = self.drone(photo_progress(result="photo_saved", _policy="wait"))
		self.drone(take_photo(cam_id=0)).wait()
		print("< Lapse Photo Started >")

	def stop_lapse_photo(self):
		'''
		Stops current time/gps lapse photos
		Should only be used when camera mode is set to eiher "time_lapse" or "gps_lapse"
		'''
		
		self.drone(stop_photo(cam_id=0)).wait()
		self.media_saved.wait()
		print("< Lapse Photo Stopped >")

	# << Recording Methods >>
	def setup_recording(self,
		mode = "standard",
		resolution = "res_dci_4k",
		framerate = "fps_24",
		hyperlapse = "ratio_15",
	):
		'''
		Prepares the drone camera for recording, and changes camera_mode to "recording"
		
		Parameters
		----------
		mode : str, optional
			the recording mode (default = "standard")
			- "standard"/"hyperlapse"/"slow_motion"/"high_framerate"
		resolution : str, optional
			the recording resolution (default = "res_dci_4k")
			- "res_dci_4k" (4K Cinema 4096x2160 | allowed fps: 24fps)
			- "res_uhd_4k" (4K UHD 3840x2160 | allowed fps: 24/25/30fps)
			- "res_1080p" (FHD 1920x1080 | allowed fps: 24/25/30/48/50/60fps)
		framerate : str, optional
			the photo file format (default = "fps_24")
			- "fps_X" (24/25/30/48/50/60)
		hyperlapse : str, optional
			The ratio at which to drop frames (default = "ratio_15")
			only used if mode is "hyperlapse"
			- "ratio_X" (15/30/60/120/240)
		'''
		
		self.drone(set_camera_mode(cam_id=0, value="recording")).wait()
		self.drone(set_recording_mode(
			cam_id = 0,
			mode = mode,
			resolution = resolution,
			framerate = framerate,
			hyperlapse = hyperlapse,
		)).wait()
		self.camera_mode = "recording"
		print("< Recording Mode >")

	def start_recording(self):
		'''
		Starts a recording
		'''
		
		self.media_saved = self.drone(recording_progress(result="stopped", _policy="wait"))
		self.drone(start_recording(cam_id=0)).wait()
		print ("< Recording Started >")

	def stop_recording(self):
		'''
		Stops current recording
		'''
		
		self.drone(stop_recording(cam_id=0)).wait()
		self.media_saved.wait()	
		print("< Recording Stopped >")

	# << Photo & Recording Download Methods >>
	def download_last_media(self):
		'''
		Downloads the last media taken
		
		Return
		----------
		download_path : str
			the location of the downloaded image
		'''
		
		# Get Photo Media ID
		media_id = self.media_saved.received_events().last().args["media_id"]
		media_info_response = requests.get(self.drone_media_api_url + media_id)
		media_info_response.raise_for_status()
	
		# Download the photo
		for resource in media_info_response.json()["resources"]:
			image_response = requests.get(self.drone_url + resource["url"], stream=True)
			download_path = os.path.join(self.download_dir, resource["resource_id"])
			image_response.raise_for_status()

			with open(download_path, "wb") as image_file:
				shutil.copyfileobj(image_response.raw, image_file)
		print("< Media Downloaded >")
		return download_path
	
	# << Stream Methods >>
	def setup_stream(self,
		value = 2,
		record = False,
		yuv_frame_processing = "None",
		yuv_frame_cb = "None",
		h264_frame_cb = "None",
		start_cb = "None",
		end_cb = "None",
		flush_cb = "None",
	):
		'''
		Prepares the drone camera for streaming, and changes camera_mode to "streaming"
		
		Parameters
		----------
		value : int or str, optional
			the streams priority values (default = 2)
			- 0 or "low_latency"
			- 1 or "high_reliability"
			- 2 or "hight_reliability_low_framerate"
		record : bool, optional
			if true the drone sends a recording of the videos (default = False)
		yuv_frame_processing : method, optional
			a callback for live video processing
			default: saves each yuv frame as an image from a frame queue
		yuv_frame_cb : method, optional
			a callack to prepare live video processing
			default: sends each yuv frame to a frame queue
		h264_frame_cb : method, optional
			a callback for h264 frame processing
			default: saves bitrate and framerate information
		start_cb : method, optional
			a callback for the start of the stream
			default: pass
		end_cb : method, optional
			a callback for the stop of the stream
			default: pass
		flush_cb: method, optional
			a callback to flush the frame queue
			default: flushes the frame queue
		'''
		
		yuv_frame_processing, yuv_frame_cb, h264_frame_cb, start_cb, end_cb, flush_cb = self.cb_helper(
			yuv_frame_processing, yuv_frame_cb, h264_frame_cb, start_cb, end_cb, flush_cb
		)	
	
		if h264_frame_cb == self.h264_frame_cb:
			self.h264_frame_stats = []
			self.h264_stats_file = open(os.path.join(self.download_dir, "h264_stats.csv"), "w+")
			self.h264_stats_writer = csv.DictWriter(
			    self.h264_stats_file, ["fps", "bitrate"]
			)
			self.h264_stats_writer.writeheader()

		self.frame_queue = queue.Queue()
		self.processing_thread = threading.Thread(target= yuv_frame_processing)
		self.renderer = None
		self.frame_counter = 0
	
		if self.drone_rtsp_port is not None:
			self.drone.streaming.server_addr = f"{self.drone_ip}:{self.drone_rtsp_port}"

		if record == True:
			self.drone.streaming.set_output_files(
			    video=os.path.join(self.download_dir, "streaming.mp4"),
			    metadata=os.path.join(self.download_dir, "streaming_metadata.json"),
			)

		self.drone.streaming.set_callbacks(
		    raw_cb = yuv_frame_cb,
		    h264_cb = h264_frame_cb,
		    start_cb = start_cb,
		    end_cb = end_cb,
		    flush_raw_cb = flush_cb,
		)
	
		self.drone(set_streaming_mode(cam_id = 0, value = value)).wait()
		self.camera_mode = "streaming"
		print("< Streaming Mode >")
	
	def cb_helper(self, yuv_frame_processing, yuv_frame_cb, h264_frame_cb, start_cb, end_cb, flush_cb):
		if yuv_frame_processing == "None": 
			yuv_frame_processing = self.yuv_frame_processing
		if yuv_frame_cb == "None":
			yuv_frame_cb = self.yuv_frame_cb
		if h264_frame_cb == "None":
			h264_frame_cb = self.h264_frame_cb
		if start_cb == "None":
			start_cb = self.start_cb
		if end_cb == "None":
			end_cb = self.end_cb
		if flush_cb == "None":
			flush_cb = self.flush_cb
		return yuv_frame_processing, yuv_frame_cb, h264_frame_cb, start_cb, end_cb, flush_cb	

	def start_stream(self):
		self.drone.streaming.start()
		self.renderer = PdrawRenderer(pdraw=self.drone.streaming)
		self.running = True
		self.processing_thread.start()
		print("< Stream Started >")

	def stop_stream(self):
		self.running = False
		self.processing_thread.join()
		if self.renderer is not None:
		    self.renderer.stop()
		assert self.drone.streaming.stop()
		print("< Stream Stopped >")

	def yuv_frame_cb(self, yuv_frame):
		"""
		This function will be called by Olympe for each decoded YUV frame.

		:type yuv_frame: olympe.VideoFrame
		"""
		
		yuv_frame.ref()
		self.frame_queue.put_nowait(yuv_frame)
		
	def yuv_frame_processing(self):
		while self.running:
			try:
				yuv_frame = self.frame_queue.get(timeout=0.1)
				self.frame_counter += 1
				
				# the VideoFrame.info() dictionary contains some useful information
				# such as the video resolution
				info = yuv_frame.info()

				height, width = (  # noqa
				    info["raw"]["frame"]["info"]["height"],
				    info["raw"]["frame"]["info"]["width"],
				)

				# yuv_frame.vmeta() returns a dictionary that contains additional
				# metadata from the drone (GPS coordinates, battery percentage, ...)

				# convert pdraw YUV flag to OpenCV YUV flag
				cv2_cvt_color_flag = {
					olympe.VDEF_I420: cv2.COLOR_YUV2BGR_I420,
					olympe.VDEF_NV12: cv2.COLOR_YUV2BGR_NV12,
				}[yuv_frame.format()]

				cv2frame = cv2.cvtColor(yuv_frame.as_ndarray(), cv2_cvt_color_flag)
				cv2.imwrite(os.path.join(self.download_dir, "test{}.jpg".format(self.frame_counter)), cv2frame)
				
			except queue.Empty:
				continue
		# You should process your frames here and release (unref) them when you're done.
		# Don't hold a reference on your frames for too long to avoid memory leaks and/or memory
		# pool exhaustion.
		yuv_frame.unref()

	def flush_cb(self, stream):
		if stream["vdef_format"] != olympe.VDEF_I420:
			return True
		while not self.frame_queue.empty():
			self.frame_queue.get_nowait().unref()
		return True
	
	def start_cb(self):
		pass
	
	def end_cb(self):
		pass
	
	def h264_frame_cb(self, h264_frame):
		"""
		This function will be called by Olympe for each new h264 frame.

		    :type yuv_frame: olympe.VideoFrame
		"""

		# Get a ctypes pointer and size for this h264 frame
		frame_pointer, frame_size = h264_frame.as_ctypes_pointer()

		# For this example we will just compute some basic video stream stats
		# (bitrate and FPS) but we could choose to resend it over an another
		# interface or to decode it with our preferred hardware decoder..

		# Compute some stats and dump them in a csv file
		info = h264_frame.info()
		frame_ts = info["ntp_raw_timestamp"]
		if not bool(info["is_sync"]):
			while len(self.h264_frame_stats) > 0:
				start_ts, _ = self.h264_frame_stats[0]
				if (start_ts + 1e6) < frame_ts:
					self.h264_frame_stats.pop(0)
				else:
			    		break
			self.h264_frame_stats.append((frame_ts, frame_size))
			h264_fps = len(self.h264_frame_stats)
			h264_bitrate = 8 * sum(map(lambda t: t[1], self.h264_frame_stats))
			self.h264_stats_writer.writerow({"fps": h264_fps, "bitrate": h264_bitrate})




from SoftwarePilot.AnafiCameraMedia import AnafiCameraMedia
from SoftwarePilot.AnafiCameraControls import AnafiCameraControls

class AnafiCamera:
	'''
	Wrapper for the Parrot Olympe camera methods splitting media and camera controls

	...
	
	Attributes
	----------
	media : AnafiCameraMedia
		the drone camera media method interface
	controls : AnafiCameraControls
		the drone camera controls method interface
	'''
	
	def __init__(self, drone_object, drone_ip, drone_rtsp_port, drone_url, download_dir):
		'''
		Parameters
		----------
		drone_object : olympe.Drone
			the drone object
		drone_ip : str
			the drone's ip_address
		drone_rtstp_port : str
			the drone's rtsp port which connects to the live stream
		drone_url : str
			the url used request to make requests from the drone
		download_dir : str
			The location drone media will be downloaded
		'''
	
		self.media = AnafiCameraMedia(drone_object, drone_ip, drone_rtsp_port, drone_url, download_dir)
		self.controls = AnafiCameraControls(drone_object)



import docker

class SoftwarePilotDocker:
	'''
	docker wrapper library for easier interaction with SoftwarePilot
	
	...
	
	Attributes
	----------
	client : docker client
		the docker client
	
	Methods
	-------
	deploy_container(image, detach, ports)
		deploys a docker container from a given image and connects to it
	connect_to_container(container_id)
		connects to an active container
	run_command(container, command)
		runs a command on a given image
	list_containers()
		prints a list of all containers
	list_image()
		prints a list of all local images
	get_container_id(image_name)
		returns the short id of an active container from a given image name
	get_container_name(image_id)
		returns the image name of an active container from a given short id
	get_container_ip(image_id)
		returns the container ip of an active container from a given short id
	print_logs(container)
		prints the logs of an active container
	stop_container(container_id)
		stops an active container from a given container id
	stop_all_containers()
		stops all active containers
	'''
	
	def __init__(self):
		self.client = docker.from_env()
	
	def deploy_container(self, image, detach = True, ports = "None"):
		'''
		deploys a docker container from a given image and connects to it
		
		Parameters
		----------
		image : str
			the docker image to be run
		detach : bool, optional
			if True the docker is detached from the terminal (default = True)
		ports : map{ computer port : docker port }
			maps the ports from the docker to the machine
			
		Return
		----------
		container : Docker Container
			the deployed docker container
		'''
		
		try:
			container = self.client.containers.run(image, detach = detach, ports = ports)
		except:
			container = self.client.containers.run(image, detach = detach)
		print ("< Container deployed >")
		return container
	
	def connect_to_container(self, container_id):
		'''
		connects to an active container
		
		Parameters
		----------
		container_id : str
			the id of the container to connect to
			
		Return
		----------
		container : Docker Container 
			the connected docker container
		'''
		
		try:
			container = self.client.containers.get(container_id)
			print ("< Container {} connected >".format(container_id))
			return container
		except:
			print("Invalid container ID")

	def run_command(self, container, command):
		'''
		runs a command on a given image
		
		Parameters
		----------
		container : docker container
			the container to run the command on
		command : str
			the command to run
		'''
		
		container.exec_run(command)
		print ("< Command running >")
	
	def get_container_id(self, image_name):
		'''
		returns the short id of an active container from a given image name
		
		Parameters
		----------
		image_name : str
			the name of the image to fetch the short id of its container
			
		Return
		----------
		id : str
			the requested container short id
		'''
		
		for c in self.client.containers.list():
			container_image_name = c.image.tags[0].split(':')[0]
			if image_name == container_image_name:
				return c.short_id
		print("Container not found")
		return "None"
	
	def get_container_name(self, container_id):
		'''
		returns the image name of an active container from a given container short id
		
		Parameters
		----------
		container_id : str
			the id of the container to fetch the name of its image
		
		Return
		----------
		name : str
			the requested image name
		'''
		
		for c in self.client.containers.list():
			if container_id == c.id:
				image_name = c.image.tags[0].split(':')[0]
				return image_name
		print("Container not found")
		return "None"
		
	def get_container_ip(self, container_id):
		'''
		returns the ip address of an active container from a given short id
		
		Parameters
		----------
		container_id : str
			the id of the container to fetch the ip of
			
		Return
		----------
		ip : str
			the requested container ip
		'''
		
		api_client = docker.APIClient(base_url='unix://var/run/docker.sock')
		try:
			container_data = api_client.inspect_container(container_id)
			return container_data['NetworkSettings']['Networks']['bridge']['IPAddress']
		except:
			print ("Invalid container id")
				
	def print_logs(self, container):
		'''
		prints the logs of an active container
		
		Parameters
		----------
		container: docker container
			the container to print the logs of
		'''
		
		try:
			logs = container.logs(stream = True)
			while (True):
				try:
					print(next(logs, None))
				except:
					break
		except:
			print("Invalid container")
		
	def stop_container(self, container_id):
		'''
		stops an active container from a given container id
		
		Parameters
		----------
		container : docker container
			the container to stop
		
		'''
		
		try:
			container = connect_to_container(container_id)
			container.stop()
			print("< Container {} stopped >".format(container_id))
		except:
			print("Invalid container ID")
	
	def stop_all_containers(self):
		'''
		stops all active containers
		'''
	
		for container in self.client.containers.list():
  			container.stop()
  			
  	#Connect to a docker client on another machine  			
  	#List local images
  	#Add connect client off the machine given exposed port and IP
  	#When getting ID fix behavior with multiple instances
  

		
		
	



import olympe
from olympe.messages.ardrone3.Piloting import (
	TakeOff,
	Landing,
	moveBy,
	moveTo,
	CancelMoveTo,
	CancelMoveBy,
)
from olympe.messages.ardrone3.PilotingState import (
	FlyingStateChanged,
	PositionChanged,
	moveToChanged,	
)

class AnafiPiloting:
	'''
	Wrapper for the Parrot Olympe flight control methods
	
	...
	
	Attributes
	----------
	drone : olympe.Drone
		the drone object
		
	Methods
	-------
	takeoff()
		initiates drone takeoff
	land()
		initiates drone landing
	move_by(x, y, z, angle, wait)
		moves the drone a given number of meters or rotates it to a set angle
	move_to(lat, lon, alt, orientation_mode, heading, wait)
		moves the drone to given waypoint or rotates it to a set angle from north
	cancel_move_by()
		cancels move_by order
	cancel_move_to
		cancels move_to order
	'''
	
	def __init__(self, drone_object):
		'''
		Parameters
		----------
		drone_object : olympe.Drone
			the drone object
		'''
		
		self.drone = drone_object	

	def takeoff(self):
		'''
		initiates drone takeoff
		'''
		
		assert self.drone(
			TakeOff()
			>> FlyingStateChanged(state = "hovering", _timeout=5)
		).wait().success()
	
	def land(self):
		'''
		Initiates drone landing
		'''
		
		assert self.drone(Landing()).wait().success()

	def move_by(self, x, y, z, angle, wait = False):
		'''
		moves the drone a given number of meters or rotates it to a set angle
		
		Parameters
		----------
		x : float
			the movement in the x axis, forwards and backwards, in meters
		y : float
			the movement in the y axis, left and right, in meters
		z : float
			the movement in the z axis, up and down, in meters
		angle : float
			the rotation in radians
		wait : bool, optional
			if true waits for completion before sending the next instruction (default = False)
		'''
		
		if wait == False:
			self.drone(moveBy(x, y, z, angle))
		else:		
			self.drone(moveBy(x, y, z, angle) >> FlyingStateChanged(state = "hovering", _timeout=5)).wait()
	
	def move_to(self, lat, lon, alt, orientation_mode = "NONE", heading = 0, wait = False):
		'''
		moves the drone to given waypoint or rotates it to a set angle from north
		
		Parameters
		----------
		lat : float
			the latitude to travel to
		lon : float
			the longitude to travel to
		alt : float
			the altitude to travel to
		orientation_mode : str, optional
			the orientation mode (default = "NONE")
			- "NONE"
			- "TO_TARGET" (looks at the direction of the given location)
			- "HEADING_START" (Changes orientation before travelling)
			- "HEADING_DURING" (Changes orientation during travel)
		heading : float, optional
			the target orientation dictated by degrees from north (default = 0)
			only used if orientation_mode is "HEADING_START" or "HEADING_DURING"
		wait : bool, optional
			if true waits for completion before sending the next instruction (default = False)
		'''
		
		if wait == False:
			self.drone(
				moveTo(
					latitude = lat,
					longitude = lon,
					altitude = alt,
					orientation_mode = orientation_mode,
					heading = heading,
				)
			)
		else:		
			self.drone(
				moveTo(
					latitude = lat,
					longitude = lon,
					altitude = alt,
					orientation_mode = orientation_mode,
					heading = heading,
				)
				>> moveToChanged(status = "DONE").wait()
			)
		
	def cancel_move_by(self):
		'''
		cancels move_by order
		'''
		
		assert self.drone(CancelMoveBy()).wait()

	def cancel_move_to(self):
		'''
		cancels move_to order
		'''
		
		assert self.drone(CancelMoveTo()).wait()



import requests
import json

class SoftwarePilotService:
	'''
	library to interact with the custom SoftwarePilot rest api service
	
	...
	
	Attributes
	----------
	ip_address : str
		the ip address to request the rest api
	port : int
		the port used to request the rest api
	url : str
		the url used to request the rest api
	
	Methods
	-------
	set_service(ip_address, port)
		initializes the variables needed to request the rest api
	get(extension)
		executes a get request to the service
	get_ip_address()
		returns the ip address to request the rest api
	get_port()
		returns the port to request the rest api
	get_input_shape()
		returns the input shape expected by the rest api
	get_output_keys()
		returns the output keys returned by the rest api
	get_download_path()
		returns the download path where images would be uploaded
	run(image)
		executes a post request to the service, including an image file and returning a json of shape output_keys
	'''
	
	def __init__(self, ip_address, port):
		'''
		Parameters
		----------
		ip_address : str
			the ip address to request the rest api
		port: str
			the port used to request the rest api
		'''
		
		self.set_service(ip_address, port)
	
	def set_service(self, ip_address, port):
		'''
		initializes the variables needed to request the rest api
		
		Parameters
		----------
		ip_address : str
			the ip address to request the rest api
		port: str
			the port used to request the rest api
		'''
		
		self.ip_address = ip_address
		self.port = port
		self.url = "http://{}:{}/".format(self.ip_address, self.port)

	def get(self, extension = ""):
		'''
		executes a get request to the service
		
		Parameters
		----------
		extension : str, optional
			parameter to make a get request to a specific extension (default = "")
			
		Return
		----------
		data : json
			the get response
		'''
		
		response = requests.get(url = (self.url+extension))
		data = response.json()
		return data

	def get_ip_address(self):
		'''
		returns the ip address to request the rest api
		
		Return
		----------
		ip_address : str
			the ip_address
		'''
		
		return self.ip_address
	
	def get_port(self):
		'''
		returns the port to request the rest api
		
		Return
		----------
		port : int
			the port
		'''
		
		return self.port
	
	def get_input_shape(self):
		'''
		returns the input shape expected by the rest api
		
		Return
		----------
		data : int[]
			the input shape
		'''
		
		data = self.get()	
		return data["input shape"]
	
	def get_output_keys(self):
		'''
		returns the output keys returned by the rest api
		
		Return
		----------
		output_keys : str[]
			the output keys
		'''
		
		data = self.get()	
		return data["output keys"]
	
	def get_download_path(self):
		'''
		returns the download path where images would be uploaded
		
		Return
		----------
		download_path : str
			the image download path
		'''
		
		data = self.get()
		return data["download path"]
	
	def run(self, image_path):
		'''
		executes a post request to the service, including an image file and returning a json of shape output_keys
		
		Parameters
		----------
		image_path : str
			path to the image to upload
		
		Return
		----------
		data : any[]
			the image post response
		'''
		
		extension = "image/"
		file = {'file': open(image_path, 'rb')}
		response = requests.post(url=(self.url+extension), files=file)
		data = response.json()
		return data



import setuptools

long_desc = open("README.md").read()
required = [
	"docker>=5.0.3",
	"opencv-python>=4.6.0.66",
	"requests>=2.22.0",
	"PySDL2>=0.9.12",
	"PyOpenGL>=3.1.6",
]

setuptools.setup(
    name="SoftwarePilot",
    version="1.0.8",
    author="Kevyn Angueira Irizarry",
    author_email="kevyn.angueira@gmail.com",
    description="SoftwarePilot is an open source middleware and API that supports aerial applications for Parrot Anafi drones",
    long_description=long_desc,
    long_description_content_type="text/markdown",
    url="https://github.com/boredbot2/SoftwarePilot",
    key_words="Parrot Anafi, Drone, Olympe",
    install_requires=required,
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
)




import olympe
from olympe.messages.camera import (
	reset_zoom,
	reset_alignment_offsets,
	set_zoom_target,
	set_alignment_offsets,
	alignment_offsets,
)

# << Camera Movementa and Zoom Methods >>
class AnafiCameraControls:
	'''
	Wrapper for the Parrot Olympe camera methods involving camera controls
	
	...
	
	Attributes
	----------
	drone : olympe.Drone
		the drone object
		
	Methods
	-------
	reset_zoom()
		resets the drone's camera zoom
	reset_alignment_offsets()
		resets the drone's camera alignment offset
	set_zoom(target, control_mode)
		sets the drone's camera zoom, must call reset_zoom before being called again
	set_alignment_offsets(yaw, pitch, roll, wait)
		sets the drone's camera alignment offset
	'''

	def __init__(self, drone_object):
		'''
		Parameters
		----------
		drone_object : olympe.Drone
			the drone object
		'''
		
		self.drone = drone_object
	
	def reset_zoom(self):
		'''
		resets the drone's camera zoom
		'''
		
		self.drone(reset_zoom(cam_id = 0)).wait()
	
	def reset_alignment_offsets(self):
		'''
		resets the drone's camera alignment offset
		'''
		
		self.drone(reset_alignment_offsets(cam_id = 0)).wait()
	
	def set_zoom(self, target, control_mode = "level"):
		'''
		sets the drone's camera zoom, must call reset_zoom before being called again
		
		Parameters
		----------
		target : float
			the level or velocity of zoom
		control_mode : str, optional
			the target interpretation (default = "level")
			- "level"/"velocity"
		'''
		
		self.drone(set_zoom_target(cam_id = 0, control_mode = control_mode, target = target)).wait()
	
	def set_alignment_offsets(self, yaw, pitch, roll, wait = False):
		'''
		sets the drone's camera alignment offset
		
		Parameters
		----------
		yaw : float
			the yaw camera offset
		pitch : float
			the pitch camera offset
		roll : float
			the roll camera offset
		wait : bool, optional
			if true waits for completion before sending the next instruction (default = False)
		'''
		
		if wait == False:
			self.drone(set_alignment_offsets(cam_id = 0, yaw = yaw, pitch = pitch, roll = roll)).wait()
		else:
			self.drone(
				set_alignment_offsets(cam_id = 0, yaw = yaw, pitch = pitch, roll = roll)
				>> alignment_offsets(cam_id = 0, 
					current_yaw = yaw,
					current_pitch = pitch,
					current_roll = roll
				)
			).wait().success()

